Package: proActiv
Title: Estimate Promoter Activity from RNA-Seq data
Version: 1.22.0
Authors@R: c(
    person(given = "Deniz",
           family = "Demircioglu",
           role = c("aut"),
           email = "ddemircioglu@gis.a-star.edu.sg",
           comment = c(ORCID = "0000-0001-7857-0407")),
    person(given = "Jonathan",
          family = "Göke",
          role = c("aut"),
          email = "gokej@gis.a-star.edu.sg"),
    person(given = "Joseph",
           family = "Lee",
           role = c("cre"),
           email = "joseph.lee@u.nus.edu"))
Description: Most human genes have multiple promoters that control the 
  expression of different isoforms. The use of these alternative promoters 
  enables the regulation of isoform expression pre-transcriptionally. 
  Alternative promoters have been found to be important in a wide number of cell
  types and diseases. proActiv is an R package that enables the analysis of 
  promoters from RNA-seq data. proActiv uses aligned reads as input, and 
  generates counts and normalized promoter activity estimates for each annotated
  promoter. In particular, proActiv accepts junction files from TopHat2 or STAR 
  or BAM files as inputs. These estimates can then be used to identify which 
  promoter is active, which promoter is inactive, and which promoters change 
  their activity across conditions. proActiv also allows visualization of 
  promoter activity across conditions.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.0.0)
RoxygenNote: 7.3.1
Imports: AnnotationDbi, BiocParallel, data.table, dplyr, DESeq2,
        IRanges, GenomicRanges, GenomicFeatures, GenomicAlignments,
        GenomeInfoDb, ggplot2, gplots, graphics, methods, rlang,
        scales, S4Vectors, SummarizedExperiment, stats, tibble,
        txdbmaker
Suggests: GenomeInfoDbData, testthat, rmarkdown, knitr, Rtsne,
        gridExtra
URL: https://github.com/GoekeLab/proActiv
biocViews: RNASeq, GeneExpression, Transcription, AlternativeSplicing,
        GeneRegulation, DifferentialSplicing, FunctionalGenomics,
        Epigenetics, Transcriptomics, Preprocessing
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/proActiv
git_branch: RELEASE_3_23
git_last_commit: b2595a3
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:57:47 UTC; biocbuild
Author: Deniz Demircioglu [aut] (ORCID:
    <https://orcid.org/0000-0001-7857-0407>),
  Jonathan Göke [aut],
  Joseph Lee [cre]
Maintainer: Joseph Lee <joseph.lee@u.nus.edu>
