# Re-exports

# AnnotationDbi

#' @importFrom AnnotationDbi loadDb
#' @export
AnnotationDbi::loadDb

# summarizedExperiment 

#' @importFrom SummarizedExperiment assays
#' @export
SummarizedExperiment::assays

#' @importFrom SummarizedExperiment rowData
#' @export
SummarizedExperiment::rowData

#' @importFrom SummarizedExperiment colData
#' @export
SummarizedExperiment::colData

# dplyr

#' @importFrom dplyr as_tibble
#' @export
dplyr::as_tibble

#' @importFrom dplyr '%>%'
#' @export
dplyr::`%>%`

#' @importFrom dplyr group_by
#' @export
dplyr::group_by

#' @importFrom dplyr mutate
#' @export
dplyr::mutate

#' @importFrom dplyr filter
#' @export
dplyr::filter

#' @importFrom dplyr n
#' @export
dplyr::n

#' @importFrom dplyr arrange
#' @export
dplyr::arrange

#' @importFrom tibble rownames_to_column
#' @export
tibble::rownames_to_column

